
package org.ossj.xml.common.v1_5;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlType;


/**
 * A base interface for every application specific <CODE>&lt;ManagedEntity&gt;KeyResult</CODE>
 *                 interface.
 *                 <p>
 *                 A <CODE>&lt;ManagedEntity&gt;KeyResult</CODE> interface must comply with the following template:
 *                 <pre>
 *                 public interface &lt;ManagedEntity&gt;KeyResult extends ManagedEntityKeyResult
 *                 {
 *                 public &lt;ManagedEntity&gt;Key get&lt;ManagedEntity&gt;Key();
 *                 }
 *                 </pre>
 *                 <p>
 *                 The managed entity key result encapsulates all the information necessary to
 *                 indicate the result of  a BEST EFFORT operation on a specific managed entity.
 *                 <p>
 *                 The managed entity key result is used in operations involving the update of
 *                 multiple managed entities, in the deletion of multiple managed entities or in
 *                 the creation of multiple managed entities or in bulk business operations.
 *                 <p>
 *                 The base <CODE>ManagedEntityKeyResult</CODE> interface contains the managed entity key,
 *                 a boolean value indicating if the operation on the targeted managed entity was
 *                 succesful and finally the exception that would have been thrown if this operation
 *             had been attempted on the individual Managed Entity.
 * 
 * <p>Java class for ManagedEntityKeyResult complex type.
 * 
 * <p>The following schema fragment specifies the expected content contained within this class.
 * 
 * <pre>
 * &lt;complexType name="ManagedEntityKeyResult">
 *   &lt;complexContent>
 *     &lt;restriction base="{http://www.w3.org/2001/XMLSchema}anyType">
 *       &lt;sequence>
 *         &lt;element name="success" type="{http://www.w3.org/2001/XMLSchema}boolean"/>
 *         &lt;element name="exception" type="{http://ossj.org/xml/Common/v1-5}BaseException" minOccurs="0"/>
 *       &lt;/sequence>
 *     &lt;/restriction>
 *   &lt;/complexContent>
 * &lt;/complexType>
 * </pre>
 * 
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "ManagedEntityKeyResult", propOrder = {
    "success",
    "exception"
})
public abstract class ManagedEntityKeyResult {

    protected boolean success;
    protected BaseException exception;

    /**
     * Gets the value of the success property.
     * 
     */
    public boolean isSuccess() {
        return success;
    }

    /**
     * Sets the value of the success property.
     * 
     */
    public void setSuccess(boolean value) {
        this.success = value;
    }

    /**
     * Gets the value of the exception property.
     * 
     * @return
     *     possible object is
     *     {@link BaseException }
     *     
     */
    public BaseException getException() {
        return exception;
    }

    /**
     * Sets the value of the exception property.
     * 
     * @param value
     *     allowed object is
     *     {@link BaseException }
     *     
     */
    public void setException(BaseException value) {
        this.exception = value;
    }

}
